@@ -4,6 +4,7 @@ from transformers.modeling_utils import Conv1D
 from rich.console import Console
 from rich.panel import Panel
 from .discovery import DISCOVERY_REGISTRY, default_discover, ATTENTION_DISCOVERY_REGISTRY
+from rich.progress import Progress, BarColumn, TimeElapsedColumn, MofNCompleteColumn
 import psutil
 import os
 
@@ -21,6 +22,7 @@ class Pruner:
         self.initial_params_num = sum(p.numel() for p in model.parameters())
         
         self._init_cpu_mem = psutil.Process(os.getpid()).memory_info().rss / 1024**2
+        
         if torch.cuda.is_available():
             device = next(model.parameters()).device
             torch.cuda.reset_peak_memory_stats(device)
@@ -68,6 +70,7 @@ class Pruner:
         total = mags.numel()
         k = int((1.0 - sparsity) * total)
         keep = torch.topk(mags, k=k).indices
+        
         return keep, total
 
 
@@ -84,8 +87,15 @@ class Pruner:
         """
         Rebuild a Linear layer by slicing its input/output weights.
         """
-        W = layer.weight.data.cpu() 
-        B = layer.bias.data.cpu() if layer.bias is not None else None
+        
+        device = layer.weight.device
+        W = layer.weight.data
+        B = layer.bias.data if layer.bias is not None else None
+
+        if keep_out is not None:
+            keep_out = keep_out.to(device)
+        if keep_in is not None:
+            keep_in = keep_in.to(device)
 
         if keep_out is not None and keep_in is None:
             new_W = W[keep_out, :]
@@ -107,8 +117,15 @@ class Pruner:
         """
         Same as _rebuild_linear but for HuggingFace's Conv1D (used in GPT-2).
         """
-        W = layer.weight.data.cpu()
-        B = layer.bias.data.cpu() if layer.bias is not None else None
+        
+        device = layer.weight.device
+        W = layer.weight.data
+        B = layer.bias.data if layer.bias is not None else None
+        
+        if keep_out is not None:
+            keep_out = keep_out.to(device)
+        if keep_in is not None:
+            keep_in = keep_in.to(device)
 
         if keep_out is not None and keep_in is None:
             new_W = W[:, keep_out]
@@ -161,61 +178,66 @@ class Pruner:
         self.model.eval()
         device = next(self.model.parameters()).device
 
-        console.rule(f"[bold]Starting Pruning at {sparsity:.0%} Sparsity")
+        console.print(f"[bold]Starting Pruning at {sparsity:.0%} Sparsity")
         blocks = Pruner._discover_mlp_blocks(self.model)
         console.print(f"[bold]Discovered {len(blocks)} MLP blocks[/bold]\n")
 
-        for i, blk in enumerate(blocks):
-            console.print(f"[bold white]Block {i}[/bold white] – Type: {blk['type']}")
+        with Progress(
+            "[progress.description]{task.description}",
+            BarColumn(),
+            MofNCompleteColumn(),
+            TimeElapsedColumn(),
+            console=console,
+            transient=True
+        ) as progress:
+            task = progress.add_task("Pruning MLP blocks", total=len(blocks))
 
-            hook_key = blk["gate_name"] if blk["type"] == "gated" else blk["fc_name"]
-            
-            self.activations = {}
-            handle = self._hook_activations(hook_key)
+            for i, blk in enumerate(blocks):
+                hook_key = blk["gate_name"] if blk["type"] == "gated" else blk["fc_name"]
+                self.activations = {}
+                handle = self._hook_activations(hook_key)
 
-            total_acts = None
-            num_batches = 0
+                total_acts = None
+                num_batches = 0
 
-            for batch in dataloader:
-                if num_batches >= max_batches:
-                    break
-                with torch.no_grad():
-                    _ = self.model(**{k: v.to(device) for k, v in batch.items()})
-                act = self.activations.pop(hook_key, None)
-                if act is None:
-                    continue
-                total_acts = act if total_acts is None else total_acts + act
-                num_batches += 1
+                for batch in dataloader:
+                    if num_batches >= max_batches:
+                        break
+                    with torch.no_grad():
+                        _ = self.model(**{k: v.to(device) for k, v in batch.items()})
+                    act = self.activations.pop(hook_key, None)
+                    if act is None:
+                        continue
+                    total_acts = act if total_acts is None else total_acts + act
+                    num_batches += 1
 
-            handle.remove()
+                handle.remove()
 
-            if num_batches == 0:
-                raise RuntimeError(f"No activations captured for block {i} ({hook_key})")
+                if num_batches == 0:
+                    raise RuntimeError(f"No activations captured for block {i} ({hook_key})")
 
-            avg_acts = total_acts / num_batches
-            keep_idx, orig = self.pruning_strategy(avg_acts, sparsity)
+                avg_acts = total_acts / num_batches
+                keep_idx, orig = self.pruning_strategy(avg_acts, sparsity)
 
-            if blk["type"] == "gated":
-                new_gate = self._rebuild(blk["gate"],   keep_out=keep_idx)
-                new_up   = self._rebuild(blk["up"],     keep_out=keep_idx)
-                new_down = self._rebuild(blk["down"],   keep_in=keep_idx)
+                if blk["type"] == "gated":
+                    new_gate = self._rebuild(blk["gate"], keep_out=keep_idx)
+                    new_up   = self._rebuild(blk["up"],   keep_out=keep_idx)
+                    new_down = self._rebuild(blk["down"], keep_in=keep_idx)
 
-                self._replace_module(blk["gate_name"], new_gate)
-                self._replace_module(blk["up_name"],   new_up)
-                self._replace_module(blk["down_name"], new_down)
+                    self._replace_module(blk["gate_name"], new_gate)
+                    self._replace_module(blk["up_name"],   new_up)
+                    self._replace_module(blk["down_name"], new_down)
 
-                console.print(f"  [green]Pruned gated MLP[/green]: {orig} → {keep_idx.numel()} units")
+                else:
+                    new_fc   = self._rebuild(blk["fc"],   keep_out=keep_idx)
+                    new_proj = self._rebuild(blk["proj"], keep_in=keep_idx)
 
-            else:
-                new_fc   = self._rebuild(blk["fc"],   keep_out=keep_idx)
-                new_proj = self._rebuild(blk["proj"], keep_in=keep_idx)
+                    self._replace_module(blk["fc_name"],   new_fc)
+                    self._replace_module(blk["proj_name"], new_proj)
 
-                self._replace_module(blk["fc_name"],   new_fc)
-                self._replace_module(blk["proj_name"], new_proj)
+                progress.advance(task)
 
-                console.print(f"  [green]Pruned FFN[/green]: {orig} → {keep_idx.numel()} units")
-
-        console.rule("[bold green]Pruning Complete")    
+        console.print("[bold green]Pruning Complete")    
 
     def report(self):
         """
@@ -268,68 +290,78 @@ class Pruner:
         """
         self.model.eval()
         device = next(self.model.parameters()).device
-        console.rule(f"[bold]Starting Attention Pruning at {sparsity:.0%} Sparsity")
+        console.print(f"[bold]Starting Attention Pruning at {sparsity:.0%} Sparsity")
 
         blocks = ATTENTION_DISCOVERY_REGISTRY.get(type(self.model).__name__, lambda m: [])(self.model)
         console.print(f"[bold]Discovered {len(blocks)} attention blocks[/bold]\n")
 
-        for blk in blocks:
-            prefix = blk["prefix"]
-            key = blk.get("qkv_name") or blk["q_name"]
-            handle = self._hook_activations(key)
-            total_acts, n = None, 0
-            for batch in dataloader:
-                if n >= max_batches:
-                    break
-                with torch.no_grad():
-                    _ = self.model(**{k: v.to(device) for k, v in batch.items()})
-                act = self.activations.pop(key, None)
-                if act is not None:
-                    total_acts = act if total_acts is None else total_acts + act
-                    n += 1
-            handle.remove()
-            if n == 0:
-                raise RuntimeError(f"No activations for attention block {prefix}")
+        with Progress(
+                "[progress.description]{task.description}",
+                BarColumn(),
+                MofNCompleteColumn(),
+                TimeElapsedColumn(),
+                console=console,
+                transient=True
+        ) as progress:
+            task = progress.add_task("Pruning Attention Blocks", total=len(blocks))
 
-            avg_act = (total_acts / n).abs()    
-            B, T, D = avg_act.shape
-            H = blk["num_heads"]
-            head_dim_qkv = D // H
+            for blk in blocks:
+                prefix = blk["prefix"]
+                key = blk.get("qkv_name") or blk["q_name"]
+                handle = self._hook_activations(key)
+                total_acts, n = None, 0
+                for batch in dataloader:
+                    if n >= max_batches:
+                        break
+                    with torch.no_grad():
+                        _ = self.model(**{k: v.to(device) for k, v in batch.items()})
+                    act = self.activations.pop(key, None)
+                    if act is not None:
+                        total_acts = act if total_acts is None else total_acts + act
+                        n += 1
+                handle.remove()
+                if n == 0:
+                    raise RuntimeError(f"No activations for attention block {prefix}")
 
-            head_mags = avg_act.view(B, T, H, head_dim_qkv).mean(dim=(0, 1, 3))
-            keep, orig = self.pruning_strategy(head_mags, sparsity)
+                avg_act = (total_acts / n).abs()    
+                B, T, D = avg_act.shape
+                H = blk["num_heads"]
+                head_dim_qkv = D // H
 
-            idx_qkv = torch.cat([
-                torch.arange(h * head_dim_qkv, (h + 1) * head_dim_qkv, device=keep.device)
-                for h in keep
-            ])
+                head_mags = avg_act.view(B, T, H, head_dim_qkv).mean(dim=(0, 1, 3))
+                keep, orig = self.pruning_strategy(head_mags, sparsity)
 
-            out_layer = blk["out"]
-            if isinstance(out_layer, Conv1D):
-                in_dim = out_layer.weight.size(0)
-            else:
-                in_dim = out_layer.in_features
-            head_dim_out = in_dim // H
-            idx_out = torch.cat([
-                torch.arange(h * head_dim_out, (h + 1) * head_dim_out, device=keep.device)
-                for h in keep
-            ])
+                idx_qkv = torch.cat([
+                    torch.arange(h * head_dim_qkv, (h + 1) * head_dim_qkv, device=keep.device)
+                    for h in keep
+                ])
 
-            if blk["type"] == "packed":
-                new_qkv = self._rebuild(blk["qkv"], keep_out=idx_qkv)
-                self._replace_module(blk["qkv_name"], new_qkv)
-            else:
-                for name in ("q", "k", "v"):
-                    new_lin = self._rebuild(blk[name], keep_out=idx_qkv)
-                    self._replace_module(blk[f"{name}_name"], new_lin)
+                out_layer = blk["out"]
+                if isinstance(out_layer, Conv1D):
+                    in_dim = out_layer.weight.size(0)
+                else:
+                    in_dim = out_layer.in_features
+                head_dim_out = in_dim // H
+                idx_out = torch.cat([
+                    torch.arange(h * head_dim_out, (h + 1) * head_dim_out, device=keep.device)
+                    for h in keep
+                ])
 
-            new_out = self._rebuild(blk["out"], keep_in=idx_out)
-            self._replace_module(blk["out_name"], new_out)
-            self._patch_attention_module(prefix, keep.numel(), head_dim_out)
+                if blk["type"] == "packed":
+                    new_qkv = self._rebuild(blk["qkv"], keep_out=idx_qkv)
+                    self._replace_module(blk["qkv_name"], new_qkv)
+                else:
+                    for name in ("q", "k", "v"):
+                        new_lin = self._rebuild(blk[name], keep_out=idx_qkv)
+                        self._replace_module(blk[f"{name}_name"], new_lin)
 
-            console.print(f"[green]Pruned {prefix}[/green]: {orig} → {keep.numel()} heads")
+                new_out = self._rebuild(blk["out"], keep_in=idx_out)
+                self._replace_module(blk["out_name"], new_out)
+                self._patch_attention_module(prefix, keep.numel(), head_dim_out)
 
-        console.rule("[bold green]Attention Pruning Complete")
+                progress.advance(task)
+
+        console.print("[bold green]Attention Pruning Complete")
 
     def _patch_attention_module(self, prefix: str, new_heads: int, head_dim_out: int):
         """
@@ -373,4 +405,67 @@ class Pruner:
         handle.remove()
         if n == 0:
             raise RuntimeError(f"No activations for {blk['prefix']}")
-        return total / n
\ No newline at end of file
+        return total / n
+
+    def _normalize_strategy_name(self, name: str) -> str:
+        n = name.lower().strip()
+        if n in {"ffn", "mlp"}:
+            return "ffn"
+        if n in {"attention", "attn"} or n.startswith("atten"):
+            return "attention"
+        raise ValueError(f"Unknown strategy '{name}'. Valid: ['ffn'|'mlp', 'attention'|'attn']")
+
+    def prune(
+        self,
+        dataloader: torch.utils.data.DataLoader,
+        strategy=("ffn",),                 
+        sparsity: float = 0.3,            
+        max_batches: int = 10,           
+        **common_kwargs,                   
+    ):
+        """
+        Unified entry point to run one or more pruning passes in order.
+
+        strategy:
+          - a string, e.g. "ffn" or "attention"
+          - a sequence of strings, e.g. ["ffn", "attention"]
+          - a sequence of (name, kwargs) pairs, e.g.
+                [("ffn", {"sparsity":0.4}), ("attention", {"sparsity":0.2, "max_batches":5})]
+
+        Notes: uses your existing activation-based criterion (selection rule) for both
+        FFN (MLP) neuron pruning and attention head pruning. (criterion = pruning strategy).
+        """
+        # normalize to a list of steps: [(name, kwargs), ...]
+        if isinstance(strategy, (str,)):
+            steps = [(self._normalize_strategy_name(strategy), {})]
+        else:
+            steps = []
+            for s in strategy:
+                if isinstance(s, (tuple, list)):
+                    name, kw = s
+                    steps.append((self._normalize_strategy_name(name), dict(kw)))
+                else:
+                    steps.append((self._normalize_strategy_name(s), {}))
+
+        def _run_ffn(**kw):
+            self.prune_all_mlp_layers(
+                dataloader=dataloader,
+                sparsity=kw.get("sparsity", sparsity),
+                max_batches=kw.get("max_batches", max_batches),
+            )
+
+        def _run_attention(**kw):
+            self.prune_attention_heads(
+                dataloader=dataloader,
+                sparsity=kw.get("sparsity", sparsity),
+                max_batches=kw.get("max_batches", max_batches),
+            )
+
+        runners = {
+            "ffn": _run_ffn,
+            "attention": _run_attention,
+        }
+
+        for name, per_step in steps:
+            merged = {**common_kwargs, **per_step}
+            runners[name](**merged)
\ No newline at end of file
